@@ -35,6 +35,7 @@ GetOptions(
     'with-mysql=s' => \my $mysql,
     'data-dir=s' => \my $data_dir,
     'log-format=s' => \my $log_format,
+    'web-max-workers=i' => \my $web_max_workers,
     "h|help" => \my $help,
 );
 
@@ -62,7 +63,7 @@ else {
     if ( ! -d $data_dir ) {
         mkpath($data_dir) or die "cannot create data directory '$data_dir': $!";
     }
-    open( my $fh, '>', "$data_dir/$$.tmp") or die 'cannot create file in data_dir: $!';
+    open( my $fh, '>', "$data_dir/$$.tmp") or die "cannot create file in data_dir: $!";
     close($fh);
     unlink("$data_dir/$$.tmp");
 }
@@ -147,7 +148,7 @@ $proclet->service(
             'Starlet',
             port => $port,
             host => $host || 0,
-            max_workers => 4,
+            max_workers => $web_max_workers || 4,
         );
         infof( "GrowthForecast::Web starts listen on %s:%s", $host || 0, $port );
         $loader->run($app);
@@ -239,6 +240,11 @@ Default is "1" (enabled)
 DB connection setting to store  metadata. format like dbi:mysql:[dbname];hostname=[hostnaem]
 Default is no mysql setting. GrowthForecast save metadata to SQLite
 
+=item --web-max-workers
+
+Number of web server processes. Default is 4
+
+
 =item -h --help
 
 Display help
